use dialoguer::{Input, MultiSelect, Select};
use crate::common::{operations, repository};
use crate::common::config::ModToml;

#[derive(Debug, Clone)]
enum MainMenu {
    Install,
    Collect,
    Pull,
    Diff,
    Upload,
    Quit
}

impl ToString for MainMenu {
    fn to_string(&self) -> String {
        match self {
            MainMenu::Install => "Install files",
            MainMenu::Collect => "Collect files",
            MainMenu::Pull => "Pull from git",
            MainMenu::Diff => "View git diff",
            MainMenu::Upload => "(Commit) and push to git",
            MainMenu::Quit => "Exit"
        }.to_string()
    }
}

pub fn select<F>(title: Option<&str>, mut options: Vec<F>) -> F
    where F: ToString
{
    let mut sel = Select::new();
    sel.items(&options);
    sel.default(0);
    if let Some(title) = title {
        sel.with_prompt(title);
    }
    let sel = sel.interact().expect("User didn't enter anything");

    options.remove(sel)
}

pub fn multi_select<F>(title: Option<&str>, options: Vec<F>) -> Vec<F>
    where F: ToString
{
    let mut sel = MultiSelect::new();
    sel.items(&options);
    if let Some(title) = title {
        sel.with_prompt(title);
    }
    let sel = sel.interact().expect("User didn't enter anything");

    options.into_iter()
        .enumerate()
        .filter_map(|(i, v)| sel.contains(&i).then_some(v))
        .collect()
}

pub fn input(title: &str) -> String {
    Input::new()
        .with_prompt(title)
        .interact_text()
        .expect("User didn't enter anything")
}

pub fn run(config: &ModToml) {
    'main_loop: loop {
        let res = select(
            Some("What do you want to do?"),
            vec![
                MainMenu::Install,
                MainMenu::Collect,
                MainMenu::Pull,
                MainMenu::Diff,
                MainMenu::Upload,
                MainMenu::Quit
            ]
        );
        match res {
            MainMenu::Install =>
                operations::install(&multi_select(
                    Some("Which modules do you want to install?"),
                    config.into()
                )),
            MainMenu::Collect =>
                operations::collect(&multi_select(
                    Some("Which modules do you want to collect?"),
                    config.into()
                )),
            MainMenu::Pull => repository::pull_repo(),
            MainMenu::Diff => repository::diff(),
            MainMenu::Upload => operations::upload(|| input("Commit message")),
            MainMenu::Quit => break 'main_loop
        }
    }
}